package com.jjt.calc.service.impl;

import com.jjt.biz.domain.TwinDevice;
import com.jjt.calc.domain.TwinCalcHour;
import com.jjt.calc.domain.TwinCalcStop;
import com.jjt.calc.domain.TwinRecordStop;
import com.jjt.calc.mapper.TwinCalcHourMapper;
import com.jjt.calc.mapper.TwinCalcStopMapper;
import com.jjt.calc.service.ITwinCalcHourService;
import com.jjt.calc.service.ITwinCalcStopService;
import com.jjt.calc.service.ITwinRecordStopService;
import com.jjt.common.utils.DateUtils;
import com.jjt.common.utils.StringUtils;
import com.jjt.common.utils.bean.BeanUtils;
import com.jjt.system.service.ISysConfigService;
import org.apache.ibatis.session.ExecutorType;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 停机数据统计Service业务层处理
 *
 * @author wukai
 * @date 2025-01-18
 */
@Service
public class TwinCalcStopServiceImpl implements ITwinCalcStopService {
    @Resource
    private TwinCalcStopMapper twinCalcStopMapper;
    @Resource
    private ISysConfigService configService;
    @Resource
    private ITwinRecordStopService stopService;
    @Resource
    private SqlSessionFactory factory;
    @Resource
    private ITwinCalcHourService calcHourService;

    /**
     * 查询停机数据统计
     *
     * @param id 停机数据统计主键
     * @return 停机数据统计
     */
    @Override
    public TwinCalcStop selectTwinCalcStopById(Long id) {
        return twinCalcStopMapper.selectTwinCalcStopById(id);
    }

    /**
     * 查询停机数据统计列表
     *
     * @param twinCalcStop 停机数据统计
     * @return 停机数据统计
     */
    @Override
    public List<TwinCalcStop> selectTwinCalcStopList(TwinCalcStop twinCalcStop) {
        return twinCalcStopMapper.selectTwinCalcStopList(twinCalcStop);
    }

    /**
     * 新增停机数据统计
     *
     * @param twinCalcStop 停机数据统计
     * @return 结果
     */
    @Override
    public int insertTwinCalcStop(TwinCalcStop twinCalcStop) {
        return twinCalcStopMapper.insertTwinCalcStop(twinCalcStop);
    }

    /**
     * 修改停机数据统计
     *
     * @param twinCalcStop 停机数据统计
     * @return 结果
     */
    @Override
    public int updateTwinCalcStop(TwinCalcStop twinCalcStop) {
        return twinCalcStopMapper.updateTwinCalcStop(twinCalcStop);
    }

    /**
     * 批量删除停机数据统计
     *
     * @param ids 需要删除的停机数据统计主键
     * @return 结果
     */
    @Override
    public int deleteTwinCalcStopByIds(Long[] ids) {
        return twinCalcStopMapper.deleteTwinCalcStopByIds(ids);
    }

    /**
     * 删除停机数据统计信息
     *
     * @param id 停机数据统计主键
     * @return 结果
     */
    @Override
    public int deleteTwinCalcStopById(Long id) {
        return twinCalcStopMapper.deleteTwinCalcStopById(id);
    }

    /**
     * 停机数据治理，从停机记录表到停机统计表中
     *
     * @param startTime  开始时间
     * @param endTime    结束时间
     * @param deviceList 设备列表
     */
    @Override
    public void process(LocalDateTime startTime, LocalDateTime endTime, List<TwinDevice> deviceList) {
        String config = configService.selectConfigByKey("data.legal.time");
        int legalTime = Integer.parseInt(config) * 1000;

        Date date = Date.from(startTime.toLocalDate().atStartOfDay(ZoneOffset.of("+8")).toInstant());
        Date startDate = DateUtils.toDate(startTime);
        Date endDate = DateUtils.toDate(endTime);
        List<TwinCalcStop> insertList = new ArrayList<>();
        List<TwinCalcStop> updateList = new ArrayList<>();
        //查询上次未统计完整的数据
        TwinCalcStop calcSearch = new TwinCalcStop();
        calcSearch.setCalcStatus("1");
        List<TwinCalcStop> oldList = selectTwinCalcStopList(calcSearch);
        Map<Long, List<TwinCalcStop>> oldMap = oldList.stream().collect(Collectors.groupingBy(TwinCalcStop::getDeviceId, LinkedHashMap::new, Collectors.toList()));
        //查询本次数据
        TwinRecordStop recordSearch = new TwinRecordStop();
        recordSearch.setDataDate(date);
        recordSearch.setHour(startTime.getHour());
        List<TwinRecordStop> newList = stopService.selectTwinRecordStopList(recordSearch);
        Map<Long, List<TwinRecordStop>> newMap = newList.stream().collect(Collectors.groupingBy(TwinRecordStop::getDeviceId, LinkedHashMap::new, Collectors.toList()));
        //查询统计数据
        TwinCalcHour hourSearch = new TwinCalcHour();
        hourSearch.setDataDate(date);
        hourSearch.setHour(startTime.getHour());
        List<TwinCalcHour> hourList = calcHourService.selectTwinCalcHourList(hourSearch);

        List<TwinCalcHour> calcStopTimeList = new ArrayList<>();
        List<TwinRecordStop> recordList = new ArrayList<>();
        //需要删除的数据
        List<Long> delIds = new ArrayList<>();
        for (TwinDevice device : deviceList) {
            //先处理上次未统计完整的数据
            List<TwinRecordStop> list = new ArrayList<>();
            if (oldMap.get(device.getDeviceId()) != null) {
                for (TwinCalcStop old : oldMap.get(device.getDeviceId())) {
                    if (!old.getEndTime().after(endDate)) {
                        TwinRecordStop stop = new TwinRecordStop();
                        BeanUtils.copyProperties(old, stop);
                        //这里存ID，方便后面需要删除，判断是否之前的记录，则直接判断remark是否为空即可
                        stop.setRemark(old.getId() + "");
                        list.add(stop);
                    }
                }
            }
            //再处理本次数据，2次结合
            if (newMap.get(device.getDeviceId()) != null) {
                list.addAll(newMap.get(device.getDeviceId()));
            }

            Map<Integer, List<TwinRecordStop>> groupList = list.stream().collect(Collectors.groupingBy(TwinRecordStop::getStopType));
            for (Map.Entry<Integer, List<TwinRecordStop>> entry : groupList.entrySet()) {
                List<TwinRecordStop> temp = entry.getValue();
                if (temp.size() == 1) {
                    //只有一条记录，则直接记录下来
                    recordList.add(temp.get(0));
                }
                TwinRecordStop lastStop = null;
                for (int i = 0; i < temp.size(); i++) {
                    TwinRecordStop currStop = temp.get(i);
                    if (lastStop == null) {
                        lastStop = currStop;
                    } else {
                        //1.第一轮，下一条开始时间-上一条结束时间小于 seconds 则两条记录合并
                        if (currStop.getStartTime().getTime() - lastStop.getEndTime().getTime() < legalTime) {
                            lastStop.setEndTime(currStop.getEndTime());
                            lastStop.setCalcStatus(currStop.getCalcStatus());
                        } else {
                            //2.判断持续时间，如果大于seconds则记录
                            if (lastStop.getEndTime().getTime() - lastStop.getStartTime().getTime() > legalTime) {
                                recordList.add(lastStop);
                            }

                            if (StringUtils.isNotEmpty(lastStop.getRemark())) {
                                //如果无效数据是未统计完成的记录，则需要删除统计里面的数据
                                delIds.add(Long.parseLong(lastStop.getRemark()));
                            }

                            lastStop = currStop;
                        }
                        if (i + 1 == temp.size()) {
                            //补一次最后一条记录,如果最后一条记录是统计未完成的，或者持续时间大于seconds的,则补录
                            if ("1".equals(lastStop.getCalcStatus()) || lastStop.getEndTime().getTime() - lastStop.getStartTime().getTime() > legalTime) {
                                recordList.add(lastStop);
                            }

                            //如果无效数据是未统计完成的记录，则需要删除统计里面的数据
                            if (StringUtils.isNotEmpty(lastStop.getRemark())) {
                                delIds.add(Long.parseLong(lastStop.getRemark()));
                            }
                        }
                    }
                }
            }
        }
        for (TwinRecordStop stop : recordList) {
            //2.复制记录
            TwinCalcStop v = new TwinCalcStop();
            BeanUtils.copyProperties(stop, v);

            TwinCalcHour calcHour = new TwinCalcHour();
            //1.计算停机时长
            if (stop.getStartTime().before(startDate)) {
                stop.setStartTime(startDate);
            }
            long stopTime = stop.getEndTime().getTime() - stop.getStartTime().getTime();
            calcHour.setDeviceId(stop.getDeviceId());
            calcHour.setCloseTime(stopTime);
            calcStopTimeList.add(calcHour);

            if (StringUtils.isNotEmpty(stop.getRemark())) {
                //需要判断 这次是否还是未处理完成的数据，如果结束时间小于数据结束时间，则完成
                if (v.getEndTime().before(endDate)) {
                    v.setCalcStatus("0");
                    v.setRemark("");
                }
                updateList.add(v);
            } else {
                v.setId(null);
                insertList.add(v);
            }
        }
        //按ID分组，并统计停机时间
        Map<Long, Long> resultMap = calcStopTimeList.stream().collect(Collectors.groupingBy(TwinCalcHour::getDeviceId, Collectors.reducing(0L, TwinCalcHour::getCloseTime, Long::sum)));
        for (TwinCalcHour hour : hourList) {
            Long closeTime = 0L;
            if (resultMap.get(hour.getDeviceId()) != null) {
                closeTime = resultMap.get(hour.getDeviceId());
            }
            closeTime = closeTime / 1000;
            Long openTime = 3600 - closeTime;
            hour.setCloseTime(closeTime);
            hour.setOpenTime(openTime);
        }

        try (SqlSession sqlSession = factory.openSession(ExecutorType.BATCH, false)) {
            TwinCalcStopMapper mapper = sqlSession.getMapper(TwinCalcStopMapper.class);
            if (insertList.size() > 0) {
                insertList.forEach(mapper::insertTwinCalcStop);
            }
            if (updateList.size() > 0) {
                updateList.forEach(mapper::updateTwinCalcStop);
            }

            TwinCalcHourMapper hourMapper = sqlSession.getMapper(TwinCalcHourMapper.class);
            hourList.forEach(hourMapper::updateTwinCalcHour);
            sqlSession.commit();
        }

        if (delIds.size() > 0) {
            deleteTwinCalcStopByIds(delIds.toArray(new Long[]{}));
        }

        //统计完成删除记录数据，避免记录超长
        stopService.deleteTwinRecordStop(date, startTime.getHour());
    }


    /**
     * 按开始和结束时间查询数据
     *
     * @param sTime 开始时间
     * @param eTime 结束时间
     * @return 查询列表
     */
    @Override
    public List<TwinCalcStop> selectTwinCalcStopListByDate(Date sTime, Date eTime) {
        TwinCalcStop stop = new TwinCalcStop();
        Map<String, Object> params = new HashMap<>(16);
        params.put("sTime", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS,sTime));
        params.put("eTime", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, eTime));
        stop.setParams(params);
        return selectTwinCalcStopList(stop);
    }

    /**
     * 按开始和结束时间查询数据(生产天）
     *
     * @param sTime 开始时间
     * @param eTime 结束时间
     * @return 查询列表
     */
    @Override
    public List<TwinCalcStop> selectTwinCalcStopListByDate(Date sTime, Date eTime, int type) {
        TwinCalcStop stop = new TwinCalcStop();
        stop.setStopType(type);
        Map<String, Object> params = new HashMap<>(16);
        params.put("sTime", sTime);
        params.put("eTime", eTime);
        stop.setParams(params);
        return selectTwinCalcStopList(stop);
    }
}
