package com.jjt.emp.service.impl;

import java.util.List;
import org.springframework.stereotype.Service;
import com.jjt.emp.mapper.TwinEmpConfigMapper;
import com.jjt.emp.domain.TwinEmpConfig;
import com.jjt.emp.service.ITwinEmpConfigService;
import javax.annotation.Resource;

/**
 * 密度价格配置Service业务层处理
 *
 * @author wukai
 * @date 2025-01-18
 */
@Service
public class TwinEmpConfigServiceImpl implements ITwinEmpConfigService {
    @Resource
    private TwinEmpConfigMapper twinEmpConfigMapper;

    /**
     * 查询密度价格配置
     *
     * @param configId 密度价格配置主键
     * @return 密度价格配置
     */
    @Override
    public TwinEmpConfig selectTwinEmpConfigByConfigId(Long configId) {
        return twinEmpConfigMapper.selectTwinEmpConfigByConfigId(configId);
    }

    /**
     * 查询密度价格配置列表
     *
     * @param twinEmpConfig 密度价格配置
     * @return 密度价格配置
     */
    @Override
    public List<TwinEmpConfig> selectTwinEmpConfigList(TwinEmpConfig twinEmpConfig) {
        return twinEmpConfigMapper.selectTwinEmpConfigList(twinEmpConfig);
    }

    /**
     * 新增密度价格配置
     *
     * @param twinEmpConfig 密度价格配置
     * @return 结果
     */
    @Override
    public int insertTwinEmpConfig(TwinEmpConfig twinEmpConfig) {
            return twinEmpConfigMapper.insertTwinEmpConfig(twinEmpConfig);
    }

    /**
     * 修改密度价格配置
     *
     * @param twinEmpConfig 密度价格配置
     * @return 结果
     */
    @Override
    public int updateTwinEmpConfig(TwinEmpConfig twinEmpConfig) {
        return twinEmpConfigMapper.updateTwinEmpConfig(twinEmpConfig);
    }

    /**
     * 批量删除密度价格配置
     *
     * @param configIds 需要删除的密度价格配置主键
     * @return 结果
     */
    @Override
    public int deleteTwinEmpConfigByConfigIds(Long[] configIds) {
        return twinEmpConfigMapper.deleteTwinEmpConfigByConfigIds(configIds);
    }

    /**
     * 删除密度价格配置信息
     *
     * @param configId 密度价格配置主键
     * @return 结果
     */
    @Override
    public int deleteTwinEmpConfigByConfigId(Long configId) {
        return twinEmpConfigMapper.deleteTwinEmpConfigByConfigId(configId);
    }
}
