package com.jjt.ws.service;

import com.jjt.ws.domain.TwinWorkshopCalc;

import java.util.Date;
import java.util.List;

/**
 * 能源统计Service接口
 *
 * @author wukai
 * @date 2025-01-17
 */
public interface ITwinWorkshopCalcService {
    /**
     * 查询能源统计
     *
     * @param calcId 能源统计主键
     * @return 能源统计
     */
    public TwinWorkshopCalc selectTwinWorkshopCalcByCalcId(Long calcId);

    /**
     * 查询能源统计列表
     *
     * @param twinWorkshopCalc 能源统计
     * @return 能源统计集合
     */
    public List<TwinWorkshopCalc> selectTwinWorkshopCalcList(TwinWorkshopCalc twinWorkshopCalc);

    /**
     * 新增能源统计
     *
     * @param twinWorkshopCalc 能源统计
     * @return 结果
     */
    public int insertTwinWorkshopCalc(TwinWorkshopCalc twinWorkshopCalc);

    /**
     * 修改能源统计
     *
     * @param twinWorkshopCalc 能源统计
     * @return 结果
     */
    public int updateTwinWorkshopCalc(TwinWorkshopCalc twinWorkshopCalc);

    /**
     * 批量删除能源统计
     *
     * @param calcIds 需要删除的能源统计主键集合
     * @return 结果
     */
    public int deleteTwinWorkshopCalcByCalcIds(Long[] calcIds);

    /**
     * 删除能源统计信息
     *
     * @param calcId 能源统计主键
     * @return 结果
     */
    public int deleteTwinWorkshopCalcByCalcId(Long calcId);

    /**
     * 能源日统计
     *
     * @param date 日期
     * @return 结果
     */
    int calc(Date date);
    /**
     * 根据日期查询
     *
     * @param wsId wsid
     * @param date 日期
     * @return 结果
     */
    List<TwinWorkshopCalc> selectTwinWorkshopCalcListByDate(Long wsId, Date date);

    /**
     * 按月查询
     *
     * @param wsId wsid
     * @param date 月份
     * @return 结果
     */
    List<TwinWorkshopCalc> selectTwinWorkshopCalcListByMonth(Long wsId, String date);
}
