package com.jjt.ws.service.impl;

import com.jjt.calc.domain.TwinCalcHourEnergy;
import com.jjt.calc.service.ITwinCalcHourEnergyService;
import com.jjt.ws.domain.TwinWorkshop;
import com.jjt.ws.domain.TwinWorkshopCalc;
import com.jjt.ws.domain.TwinWorkshopEnergy;
import com.jjt.ws.mapper.TwinWorkshopCalcMapper;
import com.jjt.ws.service.ITwinWorkshopCalcService;
import com.jjt.ws.service.ITwinWorkshopService;
import org.apache.ibatis.session.ExecutorType;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 能源统计Service业务层处理
 *
 * @author wukai
 * @date 2025-01-17
 */
@Service
public class TwinWorkshopCalcServiceImpl implements ITwinWorkshopCalcService {
    @Resource
    private TwinWorkshopCalcMapper twinWorkshopCalcMapper;
    @Resource
    private ITwinCalcHourEnergyService energyService;
    @Resource
    private SqlSessionFactory factory;
    @Resource
    private ITwinWorkshopService wsService;

    /**
     * 查询能源统计
     *
     * @param calcId 能源统计主键
     * @return 能源统计
     */
    @Override
    public TwinWorkshopCalc selectTwinWorkshopCalcByCalcId(Long calcId) {
        return twinWorkshopCalcMapper.selectTwinWorkshopCalcByCalcId(calcId);
    }

    /**
     * 查询能源统计列表
     *
     * @param twinWorkshopCalc 能源统计
     * @return 能源统计
     */
    @Override
    public List<TwinWorkshopCalc> selectTwinWorkshopCalcList(TwinWorkshopCalc twinWorkshopCalc) {
        return twinWorkshopCalcMapper.selectTwinWorkshopCalcList(twinWorkshopCalc);
    }

    /**
     * 新增能源统计
     *
     * @param twinWorkshopCalc 能源统计
     * @return 结果
     */
    @Override
    public int insertTwinWorkshopCalc(TwinWorkshopCalc twinWorkshopCalc) {
        return twinWorkshopCalcMapper.insertTwinWorkshopCalc(twinWorkshopCalc);
    }

    /**
     * 修改能源统计
     *
     * @param twinWorkshopCalc 能源统计
     * @return 结果
     */
    @Override
    public int updateTwinWorkshopCalc(TwinWorkshopCalc twinWorkshopCalc) {
        return twinWorkshopCalcMapper.updateTwinWorkshopCalc(twinWorkshopCalc);
    }

    /**
     * 批量删除能源统计
     *
     * @param calcIds 需要删除的能源统计主键
     * @return 结果
     */
    @Override
    public int deleteTwinWorkshopCalcByCalcIds(Long[] calcIds) {
        return twinWorkshopCalcMapper.deleteTwinWorkshopCalcByCalcIds(calcIds);
    }

    /**
     * 删除能源统计信息
     *
     * @param calcId 能源统计主键
     * @return 结果
     */
    @Override
    public int deleteTwinWorkshopCalcByCalcId(Long calcId) {
        return twinWorkshopCalcMapper.deleteTwinWorkshopCalcByCalcId(calcId);
    }

    /**
     * 能源日统计
     *
     * @param date 日期
     * @return 结果
     */
    @Override
    public int calc(Date date) {
        //先删除
        twinWorkshopCalcMapper.deleteTwinWorkshopCalcByDate(date);
        List<TwinWorkshopCalc> calcList = new ArrayList<>();
        List<TwinWorkshop> wsList = wsService.selectTwinWorkshopList(new TwinWorkshop());
        Map<Long, Long> wsMap = new HashMap<>(16);
        wsList.forEach(ws -> {
            List<TwinWorkshopEnergy> list = wsService.selectTwinWorkshopByWsId(ws.getWsId()).getTwinWorkshopEnergyList();
            list.forEach(o -> wsMap.put(o.getEnergyId(), ws.getWsId()));
        });
        //获取统计数据
        List<TwinCalcHourEnergy> energyList = energyService.selectTwinEmpCalcListByDate(date);
        energyList.forEach(obj -> {
            obj.setTeam();
            obj.setWsId(wsMap.get(obj.getEnergyId()));
        });
        //先按ID分组
        Map<Long, List<TwinCalcHourEnergy>> calcMap = energyList.stream().collect(Collectors.groupingBy(o -> o.getWsId(), LinkedHashMap::new, Collectors.toList()));
        for (Long wsId : calcMap.keySet()) {
            //按班组统计
            Map<String, BigDecimal> resultMap = calcMap.get(wsId).stream().collect(Collectors.groupingBy(TwinCalcHourEnergy::getTeam, Collectors.reducing(BigDecimal.ZERO, TwinCalcHourEnergy::getDataValue, BigDecimal::add)));
            TwinWorkshopCalc calc = new TwinWorkshopCalc();
            calc.setWsId(wsId);
            calc.setDataDate(date);
            calc.setAValue(resultMap.get("A"));
            calc.setBValue(resultMap.get("B"));
            calcList.add(calc);

        }
        if (calcList.size() > 0) {
            try (SqlSession sqlSession = factory.openSession(ExecutorType.BATCH, false)) {
                TwinWorkshopCalcMapper mapper = sqlSession.getMapper(TwinWorkshopCalcMapper.class);
                calcList.forEach(mapper::insertTwinWorkshopCalc);
                sqlSession.commit();
            }
        }
        return 1;
    }

    /**
     * 根据日期查询
     *
     * @param wsId wsid
     * @param date 日期
     * @return 结果
     */
    @Override
    public List<TwinWorkshopCalc> selectTwinWorkshopCalcListByDate(Long wsId, Date date) {
        TwinWorkshopCalc search = new TwinWorkshopCalc();
        search.setWsId(wsId);
        search.setDataDate(date);
        return selectTwinWorkshopCalcList(search);
    }

    /**
     * 按月查询
     *
     * @param wsId  wsid
     * @param month 月份
     * @return 结果
     */
    @Override
    public List<TwinWorkshopCalc> selectTwinWorkshopCalcListByMonth(Long wsId, String month) {
        return twinWorkshopCalcMapper.selectTwinWorkshopCalcListByMonth(wsId, month);
    }
}
